///////////////////////////////////////////////////////////
////// GibPascal pascal_X11.c
////// Released under the GPLv3
////// Written by:
//////    Michael Gibson
//////    (c)2003-2021
///////////////////////////////////////////////////////////

#include "pascal_X11.h"

#ifdef LIN64
#ifdef X11WIN 
#include <X11/xpm.h>
#endif
#endif

#include <stdio.h>
#include "warning.xpm"
#include "information.xpm"

XWMHints        xwmh = {
    (InputHint|StateHint),        // flags 
    False,                        // input
    NormalState,                  // initial_state 
    0,                            // icon pixmap 
    0,                            // icon window 
    0, 0,                         // icon location 
    0,                            // icon mask 
    0,                            // Window group 
};

//////////////////////////////////////////////////////////
////// x_open();
//////////////////////////////////////////////////////////
void x_open()
{

    if ((display = XOpenDisplay(NULL)) == NULL) {
        printf("G Pascal: can't open %s \n", XDisplayName(NULL));
        exit(1);
    }

}
////////////////////////////////////////////////////////////
////// x_close()
////////////////////////////////////////////////////////////
void x_close()
{
  XCloseDisplay(display);
}
////////////////////////////////////////////////////////////
//////
///////////////////////////////////////////////////////////
void x_createwindow(char *command)
{
  unsigned long   bd = X_DGREY;
  unsigned long   fg = X_BLACK;
  unsigned long   bg = X_GREY;
  unsigned long   bw = 1;
  XSizeHints  xsh;

  xsh.flags = (PPosition | PSize);
  xsh.height = 200;
  xsh.width = 200;
  xsh.x = (DisplayWidth(display, DefaultScreen(display)) - xsh.width) / 2;
  xsh.y = (DisplayHeight(display, DefaultScreen(display)) - xsh.height) / 2 + 50;
  window = XCreateSimpleWindow(display,DefaultRootWindow(display),xsh.x,xsh.y,xsh.width,xsh.height,bw,bd,bg);

}
////////////////////////////////////////////////
////// x_msgbox()
///////////////////////////////////////////////
void x_msgbox(char *command)
{

    GC          pen1;              // GC to draw with -- white pen 
    GC          pen2;              // GC to draw with -- grey pen
    GC          pen3;              // GC to draw with -- dark grey pen
    GC          pen4;              // GC to draw with -- yellow pen
    GC          pen5;              // GC to draw with -- black pen
    GC          gcpixmap;
    XGCValues   pen1_prop;         // Struct for creating GC
    XGCValues   pen2_prop;         // Struct for creating GC 
    XGCValues   pen3_prop;         // Struct for creating GC
    XGCValues   pen4_prop;         // Struct for creating GC
    XGCValues   pen5_prop;         // Struct for creating GC
    XGCValues   pixmap_prop;
    Pixmap      pixmap,mask;
    XpmAttributes attributes;
    XpmColorSymbol xpmcolor[1];
    XFontStruct *fontstruct;       // Font descriptor -- all pens given same font 
    unsigned long fth;             // Font size parameters 
    unsigned long fg, bg, bd;      // Pixel values 
    unsigned long bw;              // Border width 
    XEvent      event;             // Event received 
    XSizeHints  xsh;               // Size hints for window manager 
    char       *geomSpec;          // Window geometry string 
    XSetWindowAttributes xswa;     // Temporary Set Window Attribute struct 
    int x, y;                      // label text positons
    int x2,x3;                     // button text positions
    char c_type[5];                // user read msgbox type
    int type;                      // user set msgbox type
    char title[256];               // user set msgboxy title
    char disp_string[256];         // user set text
    char font[256];                // user set font
    char c_padding[5];             // user read padding 
    int padding;                   // user set padding 
    int loop=0,loop2=0;

    do{
      loop++;
    }while(command[loop]!='(');
    loop++;

    do{                            // read type 0=plain,1=plain-ok,2=smile-ok,3=smile,4=warn-ok
      c_type[loop2]=command[loop];
      loop++;
      loop2++;
    }while(command[loop]!=',');
    c_type[loop2]='\0';
    type=atoi(c_type);
    if(strcmp(c_type,"DEFAULT")==0)type=1;
    loop++;
    loop2=0;

    do{
      disp_string[loop2]=command[loop];
      loop++;
      loop2++;
    }while(command[loop]!=',');
    disp_string[loop2]='\0';
    loop2=0;
    loop++;

    do{
      title[loop2]=command[loop];
      loop++;
      loop2++;
    }while(command[loop]!=',');
    title[loop2]='\0';
    loop2=0;
    loop++;

    do{
      font[loop2]=command[loop];    
      loop++;
      loop2++;
    }while(command[loop]!=',');    
    font[loop2]='\0';
    loop2=0;
    loop++;
    if(strcmp(font,"DEFAULT")==0)strcpy(font,"fixed");

    do{
      c_padding[loop2]=command[loop];
      loop++;
      loop2++;
    }while(command[loop]!=')');
    c_padding[loop2]='\0';
    padding=atoi(c_padding);
    if(strcmp(c_padding,"DEFAULT")==0)padding=40;

    if ((fontstruct = XLoadQueryFont(display, font)) == NULL) {
        printf("G Pascal: display %s doesn't know the font '%s'\n", DisplayString(display), font);
        exit(1);
    }
    fth = fontstruct->max_bounds.ascent + fontstruct->max_bounds.descent;

    bd = X_DGREY;
    fg = X_BLACK;
    bg = X_GREY;
    bw = 1;

    xsh.flags = (PPosition | PSize);
    xsh.height = fth + padding * 2;
    xsh.width = XTextWidth(fontstruct, disp_string, strlen(disp_string)) + padding * 2;
    xsh.x = (DisplayWidth(display, DefaultScreen(display)) - xsh.width) / 2;
    xsh.y = (DisplayHeight(display, DefaultScreen(display)) - xsh.height) / 2 + 50;
    window = XCreateSimpleWindow(display,DefaultRootWindow(display),xsh.x,xsh.y,xsh.width,xsh.height,bw,bd,bg);
    XSetStandardProperties(display, window, title, title, None, NULL, 0, &xsh);
    xswa.colormap = DefaultColormap(display, DefaultScreen(display));
    xswa.bit_gravity = CenterGravity;
    XChangeWindowAttributes(display, window, (CWColormap | CWBitGravity), &xswa);

    // set all pens to same font
    pen1_prop.font = fontstruct->fid;
    pen2_prop.font = fontstruct->fid;
    pen3_prop.font = fontstruct->fid;
    pen4_prop.font = fontstruct->fid;
    pen5_prop.font = fontstruct->fid;

    //set drawing color
    pen1_prop.foreground = X_WHITE;
    pen2_prop.foreground = X_GREY;
    pen3_prop.foreground = X_DGREY;
    pen4_prop.foreground = X_YELLOW;
    pen5_prop.foreground = X_BLACK;

    pen1_prop.background = X_GREY;
    pen2_prop.background = X_GREY;
    pen3_prop.background = X_GREY;
    pen4_prop.background = X_GREY;
    pen5_prop.background = X_GREY;

    pen1 = XCreateGC(display, window, (GCFont | GCForeground | GCBackground), &pen1_prop);
    pen2 = XCreateGC(display, window, (GCFont | GCForeground | GCBackground), &pen2_prop);
    pen3 = XCreateGC(display, window, (GCFont | GCForeground | GCBackground), &pen3_prop);
    pen4 = XCreateGC(display, window, (GCFont | GCForeground | GCBackground), &pen4_prop);
    pen5 = XCreateGC(display, window, (GCFont | GCForeground | GCBackground), &pen5_prop);
 

    if (type==4||type==5){
      attributes.valuemask = XpmColormap;
      attributes.x_hotspot = 0;
      attributes.y_hotspot = 0;
      attributes.depth = DefaultDepth(display, DefaultScreen(display));;
      attributes.colormap = DefaultColormap(display, DefaultScreen(display));
      pixmap_prop.function = GXcopy;
      pixmap_prop.plane_mask = AllPlanes;
      pixmap_prop.foreground = 0;
      gcpixmap = XCreateGC(display, window, GCForeground|GCFunction|GCPlaneMask, &pixmap_prop);
      attributes.valuemask = XpmSize;
      if(type==4)XpmCreatePixmapFromData(display, window, warning_xpm, &pixmap, &mask, &attributes);
      if(type==5)XpmCreatePixmapFromData(display, window, information_xpm, &pixmap, &mask, &attributes);
      XSetFillStyle(display, gcpixmap, FillTiled);
      XSetTile(display, gcpixmap, pixmap);
      XSetClipMask(display,gcpixmap,mask);
    }

    XSelectInput(display, window, EVENT_MASK);
    XMapWindow(display, window); //show window

    while (1) { //Loop forever,  examining each event.
        XNextEvent(display, &event);
        if (event.type == ButtonPress){
             XDestroyWindow(display,window);
             break;
        }
        if (event.type == Expose && event.xexpose.count == 0) {
            XWindowAttributes xwa;       // Temp Get Window Attribute struct 
            while (XCheckTypedEvent(display, Expose, &event));
            if (XGetWindowAttributes(display, window, &xwa) == 0)
                break;
            x = (xwa.width - XTextWidth(fontstruct, disp_string, strlen(disp_string))) / 2;
            x2= (xwa.width - XTextWidth(fontstruct, "OK", 2)) / 2;
            y = (xwa.height + fontstruct->max_bounds.ascent
                 - fontstruct->max_bounds.descent) / 2;
            XClearWindow(display, window);

            XDrawString(display, window, pen5, x, y, disp_string, strlen(disp_string));

            if (type==4||type==5){
              XFillRectangle(display, window, gcpixmap, 0,0,40,40);
            } 
            if (type==2 || type==3){
              XFillArc(display, window, pen4, 10, 10,30,30,0,360*64);
              XDrawArc(display, window, pen5, 10, 10,30,30,0,360*64);
              XFillArc(display, window, pen5, 17, 15,5,5,0,360*64);
              XFillArc(display, window, pen5, 28, 15,5,5,0,360*64);
              XDrawArc(display, window, pen5, 15, 15,20,20,0,-180*64);
            }
            if (type==1 || type==2|| type==4 || type==5){ //create ok button
              XFillRectangle(display, window, pen3, (xwa.width/2)-25 ,(xwa.height-40),50,30);
              XDrawLine(display,window,pen1,(xwa.width/2)-24,(xwa.height-39),(xwa.width/2)+24,(xwa.height-39));
              XDrawLine(display,window,pen1,(xwa.width/2)-23,(xwa.height-38),(xwa.width/2)+23,(xwa.height-38));
              XDrawLine(display,window,pen1,(xwa.width/2)-24,(xwa.height-39),(xwa.width/2)-24,(xwa.height-12));
              XFillRectangle(display, window, pen2, (xwa.width/2)-23 ,(xwa.height-37),46,24);
              XDrawString(display, window, pen5, x2 ,(xwa.height-20), "OK", 2);
            }   
        }
    }
}

////////////////////////////////////////////////
////// x_radiobox()
///////////////////////////////////////////////
void x_radiobox(char *command)
{
                                                                                                                             
    GC          pen1;              // GC to draw with -- white pen
    GC          pen2;              // GC to draw with -- black pen
    GC          pen3; 
    XGCValues   pen1_prop;         // Struct for creating GC
    XGCValues   pen2_prop;         // Struct for creating GC
    XGCValues   pen3_prop; 
    XFontStruct *fontstruct;       // Font descriptor -- all pens given same font
    unsigned long fth;             // Font size parameters
    unsigned long fg, bg, bd;      // Pixel values
    unsigned long bw;              // Border width
    XEvent      event;             // Event received
    XSizeHints  xsh;               // Size hints for window manager
    char       *geomSpec;          // Window geometry string
    XSetWindowAttributes xswa;     // Temporary Set Window Attribute struct
    int xbutton, ybutton;   
    int x, y;                      // label text positons
    int x2,x3;                     // radio text positions
    char radio1[25];               // user read msgbox type
    char radio2[25];
    char title[256];               // user set msgbox title
    char disp_string[256];         // user set text
    char font[256];                // user set font
    char c_padding[5];             // user read padding
    int padding;                   // user set padding
    int loop=0,loop2=0;
                                                                                                                             
    do{
      loop++;
    }while(command[loop]!='(');
    loop++;
                                                                                                                             
    do{                          
      radio1[loop2]=command[loop];
      loop++;
      loop2++;
    }while(command[loop]!=',');
    radio1[loop2]='\0';
    loop++;
    loop2=0;
                                
    do{
      radio2[loop2]=command[loop];
      loop++;
      loop2++;
    }while(command[loop]!=',');
    radio2[loop2]='\0';
    loop++;
    loop2=0;
                                                                                             
    do{
      disp_string[loop2]=command[loop];
      loop++;
      loop2++;
    }while(command[loop]!=',');
    disp_string[loop2]='\0';
    loop2=0;
    loop++;
                                                                                                                             
    do{
      title[loop2]=command[loop];
      loop++;
      loop2++;
    }while(command[loop]!=',');
    title[loop2]='\0';
    loop2=0;
    loop++;

    do{
      font[loop2]=command[loop];
      loop++;
      loop2++;
    }while(command[loop]!=',');
    font[loop2]='\0';
    loop2=0;
    loop++;
    if(strcmp(font,"DEFAULT")==0)strcpy(font,"fixed");
                                                                                                                             
    do{
      c_padding[loop2]=command[loop];
      loop++;
      loop2++;
    }while(command[loop]!=')');
    c_padding[loop2]='\0';
    padding=atoi(c_padding);
    if(strcmp(c_padding,"DEFAULT")==0)padding=40;
                                                                                                                             
    if ((fontstruct = XLoadQueryFont(display, font)) == NULL) {
        printf("G Pascal: display %s doesn't know the font '%s'\n", DisplayString(display), font);
        exit(1);
    }
    fth = fontstruct->max_bounds.ascent + fontstruct->max_bounds.descent;
                                                                                                                             
    bd = X_DGREY;
    fg = X_BLACK;
    bg = X_GREY;
    bw = 1;
                                                                                                                             
    xsh.flags = (PPosition | PSize);
    xsh.height = fth + padding * 2;
    xsh.width = XTextWidth(fontstruct, disp_string, strlen(disp_string)) + padding * 2;
    xsh.x = (DisplayWidth(display, DefaultScreen(display)) - xsh.width) / 2;
    xsh.y = (DisplayHeight(display, DefaultScreen(display)) - xsh.height) / 2 + 50;
    window = XCreateSimpleWindow(display,DefaultRootWindow(display),xsh.x,xsh.y,xsh.width,xsh.height,bw,bd,bg);
    XSetStandardProperties(display, window, title, title, None, NULL, 0, &xsh);
    xswa.colormap = DefaultColormap(display, DefaultScreen(display));
    xswa.bit_gravity = CenterGravity;
    XChangeWindowAttributes(display, window, (CWColormap | CWBitGravity), &xswa);
                                                                                                                             
    // set all pens to same font
    pen1_prop.font = fontstruct->fid;
    pen2_prop.font = fontstruct->fid;
    pen3_prop.font = fontstruct->fid;
                                                                                                                             
    //set drawing color
    pen1_prop.foreground = X_WHITE;
    pen2_prop.foreground = X_BLACK;
    pen3_prop.foreground = X_DGREY;
 
    pen1_prop.background = X_GREY;
    pen2_prop.background = X_GREY;
    pen3_prop.background = X_GREY;                                                                                                                         
    pen1 = XCreateGC(display, window, (GCFont | GCForeground | GCBackground), &pen1_prop);
    pen2 = XCreateGC(display, window, (GCFont | GCForeground | GCBackground), &pen2_prop);
    pen3 = XCreateGC(display, window, (GCFont | GCForeground | GCBackground), &pen3_prop);                                                                                                                         
    XSelectInput(display, window, EVENT_MASK);
    XMapWindow(display, window); //show window

    while (1) { //Loop forever,  examining each event.
        XNextEvent(display, &event);
        if (event.type == ButtonPress){
             xbutton=event.xbutton.x;
             ybutton=event.xbutton.y;
             if(ybutton <= y+20 && ybutton >= y+10) {
                X11return=1;
                XDestroyWindow(display,window);
                break;
             }
             if(ybutton <= y+40 && ybutton >= y+30) { 
                X11return=2;
                XDestroyWindow(display,window);
                break;
             }
        }
        if (event.type == Expose && event.xexpose.count == 0) {
            XWindowAttributes xwa;       // Temp Get Window Attribute struct
            while (XCheckTypedEvent(display, Expose, &event));
            if (XGetWindowAttributes(display, window, &xwa) == 0) break;

            //find placement
            x = (xwa.width - XTextWidth(fontstruct, disp_string, strlen(disp_string))) / 2;
            x2= (xwa.width - XTextWidth(fontstruct, radio1     , strlen(radio1     ))) / 2;
            x3= (xwa.width - XTextWidth(fontstruct, radio2     , strlen(radio2     ))) / 2;
            y = (xwa.height + fontstruct->max_bounds.ascent
                 - fontstruct->max_bounds.descent) / 2;
            XClearWindow(display, window);

            //check to see which radio will get aligned with which. 
            if(x2<<x3)x3=x2;
            if(x3<<x2)x2=x3;                            

            //Draw label                                                                                     
            XDrawString(display, window, pen2, x, y, disp_string, strlen(disp_string));

            //Draw radios
            XFillArc(display, window, pen3, x2-21, y+4, 15,15,0,360*64);
            XFillArc(display, window, pen1, x2-20, y+5, 15,15,0,360*64);
            XDrawString(display, window, pen2, x2, y+20, radio1, strlen(radio1 ));
            XFillArc(display, window, pen3, x3-21, y+24 ,15,15,0,360*64);
            XFillArc(display, window, pen1, x3-20, y+25 ,15,15,0,360*64); 
            XDrawString(display, window, pen2, x3, y+40, radio2, strlen(radio2 ));
        }
    }
}
////////////////////////////////////////////////////////
//////
////////////////////////////////////////////////////////

void x_imgbox(char *command)

{
    GC   pen[500];            // GC to draw with -- black pen
    GC   gcP;
    Pixmap mask, pm, pmP;
    Drawable drawpm;
    XpmAttributes attributes;
    XpmColorSymbol xpmcolor[1];
    XGCValues pen_prop;       // Struct for creating GC
    XGCValues PM_prop;
    XFontStruct *fontstruct;       // Font descriptor -- all pens given same font
    unsigned long fth;             // Font size parameters
    unsigned long fg, bg, bd;      // Pixel values
    unsigned long bw;              // Border width
    XEvent      event;             // Event received
    XSizeHints  xsh;               // Size hints for window manager
    char       *geomSpec;          // Window geometry string
    XSetWindowAttributes xswa;     // Temporary Set Window Attribute struct
    int loop=0,loop2=0;
    char filename[50],title[50];
    char is_gpg[30];
    char *pppp;
    FILE *gfile; 
    int typedraw[500], numlines=0;
    int imageh=20,imagew=20;
    unsigned long color[500];
    char c_color[500][10];
    unsigned long b_color=X_GREY;
    char bc_color[10];
    int x1[500],x2[500],y1[500],y2[500];
                                                                                                                         
    do{
      loop++;
    }while(command[loop]!='(');
    loop++;
                                                                                                                             
    do{
      filename[loop2]=command[loop];
      loop++;
      loop2++;
    }while(command[loop]!=',');
    filename[loop2]='\0';
    loop++;
    loop2=0;
   
    do{
      title[loop2]=command[loop];
      loop++;
      loop2++;
    }while(command[loop]!=')');
    title[loop2]='\0';
 
    gfile = fopen(filename,"r");

    fscanf(gfile,"%s",is_gpg);
    if(strncmp(is_gpg,"gpg",3)==0){ 
      fscanf(gfile,"%s%d%d%d",bc_color,&numlines,&imageh,&imagew);
      b_color = strtoul(bc_color,NULL,16);
      for(loop=0;loop<numlines;loop++){
        fscanf(gfile,"%d%s%d%d%d%d",&typedraw[loop],c_color[loop],&x1[loop],&y1[loop],&x2[loop],&y2[loop]);
        color[loop]= strtoul(c_color[loop],NULL,16);
      }
   }
    fclose(gfile);


    if ((fontstruct = XLoadQueryFont(display, "fixed")) == NULL) {
        printf("G Pascal: display doesn't know the font 'fixed'\n");
        exit(1);
    }
    fth = fontstruct->max_bounds.ascent + fontstruct->max_bounds.descent;

    xsh.flags = (PPosition | PSize);
    window = XCreateSimpleWindow(display,DefaultRootWindow(display),50,50,imagew,imageh,1,X_DGREY,b_color);
    XSetStandardProperties(display, window, title, title, None, NULL, 0, &xsh);
    xswa.colormap = DefaultColormap(display, DefaultScreen(display));
    xswa.bit_gravity = CenterGravity;
    XChangeWindowAttributes(display, window, (CWColormap | CWBitGravity), &xswa);
    for(loop=0;loop<numlines;loop++){
      pen_prop.font = fontstruct->fid;
      pen_prop.foreground = color[loop];
      pen_prop.background = b_color;
      pen[loop] = XCreateGC(display, window, (GCFont | GCForeground | GCBackground), &pen_prop);
    }

   if(strncmp(is_gpg,"gpg",3)!=0){
     attributes.valuemask = XpmColormap;
     attributes.x_hotspot = 0;
     attributes.y_hotspot = 0;
     attributes.depth = DefaultDepth(display, DefaultScreen(display));;
     attributes.colormap = DefaultColormap(display, DefaultScreen(display)); 
     PM_prop.function = GXcopy;
     PM_prop.plane_mask = AllPlanes;
     PM_prop.foreground = 0;
     gcP = XCreateGC(display, window, GCForeground|GCFunction|GCPlaneMask, &PM_prop);
     attributes.valuemask = XpmSize;
     XpmReadFileToPixmap(display,window,filename,&pmP,&mask,&attributes);
     XSetFillStyle(display, gcP, FillTiled);
     XSetTile(display, gcP, pmP);
     XSetClipMask(display,gcP,mask);
     imagew=attributes.width;
     imageh=attributes.height;
     XResizeWindow(display, window, imagew, imageh);
   }
    XSelectInput(display, window, EVENT_MASK);
    XMapWindow(display, window); //show window

    while (1) { //Loop forever,  examining each event.
        XNextEvent(display, &event);
        if (event.type == ButtonPress){
             XDestroyWindow(display,window);
             break;
        }
        if (event.type == Expose && event.xexpose.count == 0) {
            XWindowAttributes xwa;       // Temp Get Window Attribute struct
            while (XCheckTypedEvent(display, Expose, &event));
            if (XGetWindowAttributes(display, window, &xwa) == 0) break;
            XClearWindow(display, window);
             if(strncmp(is_gpg,"gpg",3)!=0){
               XFlush(display);
               XFillRectangle(display, window, gcP, 0,0,attributes.width + 2,attributes.height +2); 
             }
             else{
               for(loop=0;loop<numlines;loop++){
                 if(typedraw[loop]==1)XDrawLine(display,window,pen[loop],x1[loop],y1[loop],x2[loop],y2[loop]);
                 if(typedraw[loop]==2)XFillRectangle(display, window, pen[loop],x1[loop],y1[loop],x2[loop],y2[loop]);
               }
             }
        }
    }
}
